@@ -12,6 +12,9 @@ from .TFCUtils import (
     step,
     LS,
     LsClass,
+    pejit,
 )
 from .MakePlot import MakePlot
 from . import Latex
+from . import BF
+from .CeSolver import CeSolver
